package ir.samaneahan.application

import android.Manifest
import android.annotation.SuppressLint
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Color
import android.net.ConnectivityManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.provider.MediaStore
import android.text.format.DateUtils
import android.view.View
import android.webkit.*
import android.widget.FrameLayout
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.text.HtmlCompat
import com.google.android.material.snackbar.Snackbar
import com.karumi.dexter.Dexter
import com.karumi.dexter.MultiplePermissionsReport
import com.karumi.dexter.PermissionToken
import com.karumi.dexter.listener.PermissionRequest
import com.karumi.dexter.listener.multi.MultiplePermissionsListener
import im.delight.android.webview.AdvancedWebView
import kotlinx.android.synthetic.main.activity_main.*
import java.io.File
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashMap

class MainActivity : AppCompatActivity() {

    private lateinit var loadingDFF: DialogLoading
    private lateinit var fragmentError: FragmentError
    private lateinit var fragmentNoNet: FragmentNoInternet
    private lateinit var snack: Snackbar
    private lateinit var aWebview: AdvancedWebView
    private var doubleBackToExitPressedOnce = false
    var isOnline = true
    var isSiteCrash = false
    var currentURL = ""
    private var imagePathCallback: ValueCallback<Array<Uri>>? = null
    private var cameraImagePath: String? = null


    companion object {
        private const val CAMERA_REQUEST_CODE = 113
        private const val REQUEST_SELECT_FILE = 13
        private const val INTENT_FILE_TYPE = "image/*"
        private const val CAMERA_PHOTO_PATH_POSTFIX = "file:"
        private const val PHOTO_NAME_POSTFIX = "JPEG_"
        private const val PHOTO_FORMAT = ".jpg"
    }


    @SuppressLint("SetJavaScriptEnabled")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)



        aWebview = webview
        aWebview.setMixedContentAllowed(false);

        /*  aWebview.setListener(this, object : AdvancedWebView.Listener {
              override fun onPageStarted(url: String?, favicon: Bitmap?) {

              }

              override fun onPageFinished(url: String?) {

              }

              override fun onPageError(errorCode: Int, description: String?, failingUrl: String?) {

              }

              override fun onDownloadRequested(
                  url: String?,
                  suggestedFilename: String?,
                  mimeType: String?,
                  contentLength: Long,
                  contentDisposition: String?,
                  userAgent: String?
              ) {

              }

              override fun onExternalPageRequest(url: String?) {

              }
          })*/
        CookieManager.getInstance().setAcceptCookie(true);

        /*  getPermission {

          }*/

        if (!::loadingDFF.isInitialized) {
            loadingDFF = DialogLoading.newInstance(object :
                DialogLoading.Interaction {
                override fun onCancel() {
                    hideFrgLoading()
                }
            })

        }

        if (!::fragmentError.isInitialized) {
            fragmentError = FragmentError.newInstance(object : FragmentError.Interaction {
                override fun onTry() {
                    if (isSiteCrash) {
                        webview.loadUrl(App.MAIN_ADDRESS, getCustomHeaders())
                        isSiteCrash = false
                    } else {
                        webview.reload()
                    }
                    showFrgLoading()
                    hideFrgError()
                }
            })
        }

        if (!::fragmentNoNet.isInitialized) {
            fragmentNoNet = FragmentNoInternet.newInstance(object : FragmentNoInternet.Interaction {
                override fun onTry() {
                    if (checkInternet()) {
                        hideFrgNoNet()
                        webview.reload()
                        showFrgLoading()
                    } else
                        showFrgNoNet()
                }
            })
        }


        showFrgLoading()
        webview.loadUrl(App.MAIN_ADDRESS, getCustomHeaders())



        webview.setLayerType(View.LAYER_TYPE_HARDWARE, null)
        webview.settings.domStorageEnabled = true
        webview.settings.setSupportZoom(false)
        webview.settings.allowFileAccess = true
        webview.settings.setAppCacheEnabled(true)
        webview.settings.pluginState = WebSettings.PluginState.ON
        webview.settings.loadsImagesAutomatically = true
        webview.settings.allowContentAccess = true
        webview.settings.javaScriptEnabled = true
        webview.settings.allowFileAccess = true

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            webview.settings.mixedContentMode = WebSettings.MIXED_CONTENT_ALWAYS_ALLOW;


        if (savedInstanceState != null) {
            webview.restoreState(savedInstanceState)
        }
        webview.settings.javaScriptEnabled = true
        webview.webViewClient = WebViewClient()
        webview.settings.loadWithOverviewMode = true;
        webview.settings.useWideViewPort = true;
        /* webview.webChromeClient = object : WebChromeClient() {

         }*/
        webview.settings.setAppCacheEnabled(true)
        webview.settings.databaseEnabled = true
        webview.settings.domStorageEnabled = true

        webview.webViewClient = object : WebViewClient() {
            override fun shouldOverrideUrlLoading(view: WebView, url: String): Boolean {
                currentURL = url
                if (url.contains("defaultwebpage.cgi")) {
                    clearCache(this@MainActivity, 0)
                    isSiteCrash = true
                    showFrgError()
                } else if (url.startsWith("tel:") ||url.startsWith("sms:") ||
                    url.contains("bahamta.com")
                    || url.contains("https://webpay.bahamta.com") || url.contains(
                        "instagram.com"
                    ) || url.contains("shaparak.ir") || url.contains(
                        "zarinpal.com"
                    ) || url.contains("maps.google.com") || url.contains("whatsapp")
                ) {
                    val i = Intent(Intent.ACTION_VIEW)
                    i.data = Uri.parse(url)
                    startActivity(i)
                } else {
                    if (checkInternet()) {
                        hideFrgError()
                        hideNoNet()
                        showFrgLoading()
                        view.loadUrl(url, getCustomHeaders())
                    } else {
                        showFrgNoNet()
                    }
                }


                return true

            }

            override fun onPageFinished(view: WebView?, url: String?) {
                super.onPageFinished(view, url)
                hideFrgLoading()
            }


            override fun onReceivedError(
                view: WebView?,
                errorCode: Int,
                description: String?,
                failingUrl: String?
            ) {
                if (checkInternet()) {
                    val intent = Intent(Intent.ACTION_VIEW, Uri.parse(failingUrl))
                    startActivity(intent)
                    //webview.goBack()
                    //showFrgError()
                } else {
                    showFrgNoNet()

                }

            }

            override fun onPageStarted(view: WebView?, url: String?, favicon: Bitmap?) {
                super.onPageStarted(view, url, favicon)
                showFrgLoading()
            }
        }


        aWebview.setDownloadListener(DownloadListener { url, userAgent, contentDisposition, mimetype, contentLength ->
            val i = Intent(Intent.ACTION_VIEW)
            i.data = Uri.parse(url)
            startActivity(i)
        })
        aWebview.setWebChromeClient(MyChrome())


        imgBack.setOnClickListener {
            if (webview.canGoBack()) {
                showFrgLoading()
                webview.goBack()
            } else
                onBackPressed()
        }

        imgRefresh.setOnClickListener {
            if (isSiteCrash) {
                hideFrgError()
                hideNoNet()
                showFrgLoading()

                webview.loadUrl(App.MAIN_ADDRESS, getCustomHeaders())
                isSiteCrash = false

            } else {
                hideFrgError()
                hideNoNet()
                showFrgLoading()

                showFrgLoading()
                webview.reload()
            }

        }


    }

    private fun getPermission(runnable: Runnable) {
        Dexter.withActivity(this)
            .withPermissions(
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.CAMERA,
                Manifest.permission.RECORD_AUDIO
            )
            .withListener(object : MultiplePermissionsListener {

                override fun onPermissionsChecked(report: MultiplePermissionsReport?) {
                    runnable.run()
                }

                override fun onPermissionRationaleShouldBeShown(
                    permissions: MutableList<PermissionRequest>?,
                    token: PermissionToken?
                ) {
                    token!!.continuePermissionRequest()
                }


            }).check()
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        aWebview.onActivityResult(requestCode, resultCode, data)
        super.onActivityResult(requestCode, resultCode, data)

        if (requestCode != REQUEST_SELECT_FILE || imagePathCallback == null) return

        var results: Array<Uri>? = null

        if (resultCode == RESULT_OK) {
            if (data == null) {
                if (cameraImagePath != null) results = arrayOf(Uri.parse(cameraImagePath))
            } else {
                val dataString = data.dataString
                if (dataString != null) results = arrayOf(Uri.parse(dataString))
            }
        }

        imagePathCallback?.onReceiveValue(results)
        imagePathCallback = null
    }


    private fun getCustomHeaders(): Map<String, String> {
        val headers: MutableMap<String, String> = HashMap()
        headers["HTTP_USER_AGENT"] = "androidx"
        return headers
    }

    private fun hideFrgLoading() {
        frmContainerLoading.visibility = View.GONE
        webview.visibility = View.VISIBLE
    }

    private fun hideFrgError() {
        frmContainerError.visibility = View.GONE
        // webview.visibility = View.VISIBLE
    }

    private fun hideNoNet() {
        frmContainerNoNet.visibility = View.GONE
        // webview.visibility = View.VISIBLE
    }

    private fun showFrgLoading() {
        try {
            supportFragmentManager.beginTransaction().replace(R.id.frmContainerLoading, loadingDFF)
                .commit();
            loadingDFF.setAutoHide()
            frmContainerLoading.visibility = View.VISIBLE
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun showFrgError() {
        supportFragmentManager.beginTransaction().replace(R.id.frmContainerError, fragmentError)
            .commit();
        frmContainerError.visibility = View.VISIBLE


    }

    private fun showFrgNoNet() {
        supportFragmentManager.beginTransaction().replace(R.id.frmContainerNoNet, fragmentNoNet)
            .commit();
        frmContainerNoNet.visibility = View.VISIBLE
    }

    private fun hideFrgNoNet() {
        frmContainerNoNet.visibility = View.GONE
    }


    override fun onBackPressed() {
        if (!webview.canGoBack() || currentURL == "https://samaneahan.com/login/") {
            if (doubleBackToExitPressedOnce) {
                overridePendingTransition(
                    R.anim.fadein,
                    R.anim.fadeout
                )
                finish()
                return
            }
            this.doubleBackToExitPressedOnce = true
            showSnack(getString(R.string.exit_title))
            Handler().postDelayed(
                { doubleBackToExitPressedOnce = false },
                App.EXIT_INTERVAL
            )
        } else {
            showFrgLoading()
            webview.goBack()

        }

    }


    private fun showSnack(str: String) {
        snack = Snackbar.make(
            mainLayout,
            HtmlCompat.fromHtml(
                "<font color=\"#ffffff\">$str</font>",
                HtmlCompat.FROM_HTML_MODE_LEGACY
            ),
            Snackbar.LENGTH_SHORT
        )
            .setAction(getString(R.string.backExit)) {
                snack.dismiss()
                overridePendingTransition(
                    R.anim.fadein,
                    R.anim.fadeout
                )
                finish()
            }.setActionTextColor(Color.YELLOW)
        snack.show()
    }

    private fun checkInternet(): Boolean {
        val connectivityManager =
            applicationContext.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val networkInfo = connectivityManager.activeNetworkInfo
        isOnline = networkInfo != null && networkInfo.isConnected

        return isOnline
    }

    //helper method for clearCache() , recursive
    //returns number of deleted files
    private fun clearCacheFolder(dir: File?, numDays: Int): Int {
        var deletedFiles = 0
        if (dir != null && dir.isDirectory()) {
            try {
                for (child in dir.listFiles()) {

                    //first delete subdirectories recursively
                    if (child.isDirectory()) {
                        deletedFiles += clearCacheFolder(child, numDays)
                    }

                    //then delete the files and subdirectories in this dir
                    //only empty directories can be deleted, so subdirs have been done first
                    if (child.lastModified() < Date().getTime() - numDays * DateUtils.DAY_IN_MILLIS) {
                        if (child.delete()) {
                            deletedFiles++
                        }
                    }
                }
            } catch (e: Exception) {
                // Log.e(FragmentActivity.TAG, String.format("Failed to clean the cache, error %s", e.message))
            }
        }
        return deletedFiles
    }


    inner class MyChrome() : WebChromeClient() {

        private var mCustomView: View? = null
        private var mCustomViewCallback: CustomViewCallback? = null
        protected var mFullscreenContainer: FrameLayout? = null
        private var mOriginalOrientation = 0
        private var mOriginalSystemUiVisibility = 0

        override fun onShowFileChooser(
            webView: WebView?,
            filePathCallback: ValueCallback<Array<Uri>>?,
            fileChooserParams: FileChooserParams?
        ): Boolean {


            imagePathCallback?.onReceiveValue(null)
            imagePathCallback = null
            imagePathCallback = filePathCallback

            val takePictureIntent = createImageCaptureIntent()

            val contentSelectionIntent = Intent(Intent.ACTION_GET_CONTENT)
            contentSelectionIntent.addCategory(Intent.CATEGORY_OPENABLE)
            contentSelectionIntent.type = INTENT_FILE_TYPE

            /*val intentArray: Array<Intent?>
            intentArray = takePictureIntent?.let { arrayOf(it) } ?: arrayOfNulls(0)

            val chooserIntent = Intent(Intent.ACTION_CHOOSER)
            chooserIntent.putExtra(Intent.EXTRA_INTENT, contentSelectionIntent)
            chooserIntent.putExtra(Intent.EXTRA_TITLE, "Choose")
            chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS, intentArray)*/

            try {
                startActivityForResult(takePictureIntent, REQUEST_SELECT_FILE)
            } catch (e: ActivityNotFoundException) {
                imagePathCallback = null
                cameraImagePath = null

                Toast.makeText(
                    this@MainActivity,
                    "Cant open file chooser",
                    Toast.LENGTH_LONG
                ).show()

                return false
            }

            return true
        }

        private fun createImageCaptureIntent(): Intent? {
            var captureImageIntent: Intent? = Intent(MediaStore.ACTION_IMAGE_CAPTURE)

            if (captureImageIntent?.resolveActivity(packageManager) != null) {
                var imageFile: File? = null

                try {
                    imageFile = createImageFile()
                    captureImageIntent.putExtra("CameraImagePath", cameraImagePath)
                } catch (ex: IOException) {
                    ex.printStackTrace()
                }

                if (imageFile != null) {
                    cameraImagePath = CAMERA_PHOTO_PATH_POSTFIX + imageFile.absolutePath
                    captureImageIntent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(imageFile))
                } else {
                    captureImageIntent = null
                }
            }

            return captureImageIntent
        }

        private fun createImageFile(): File? {
            val timeStamp = SimpleDateFormat.getDateInstance().format(Date())
            val imageFileName = PHOTO_NAME_POSTFIX + timeStamp + "_"
            val storageDir = getExternalFilesDir(Environment.DIRECTORY_PICTURES)

            return File.createTempFile(imageFileName, PHOTO_FORMAT, storageDir)
        }

        override fun onGeolocationPermissionsShowPrompt(
            origin: String,
            callback: GeolocationPermissions.Callback
        ) {
            callback.invoke(origin, true, false)
        }

        override fun getDefaultVideoPoster(): Bitmap? {
            return if (mCustomView == null) {
                null
            } else BitmapFactory.decodeResource(getApplicationContext().getResources(), 2130837573)
        }

        override fun onHideCustomView() {
            (getWindow().getDecorView() as FrameLayout).removeView(mCustomView)
            mCustomView = null
            getWindow().getDecorView().setSystemUiVisibility(mOriginalSystemUiVisibility)
            setRequestedOrientation(mOriginalOrientation)
            mCustomViewCallback!!.onCustomViewHidden()
            mCustomViewCallback = null
        }

        override fun onShowCustomView(
            paramView: View?,
            paramCustomViewCallback: CustomViewCallback?
        ) {
            if (mCustomView != null) {
                onHideCustomView()
                return
            }
            mCustomView = paramView
            mOriginalSystemUiVisibility = window.decorView.systemUiVisibility
            mOriginalOrientation = requestedOrientation
            mCustomViewCallback = paramCustomViewCallback
            (window.decorView as FrameLayout).addView(mCustomView, FrameLayout.LayoutParams(-1, -1))
            window.decorView.systemUiVisibility = 3846 or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
        }
    }


    /*
 * Delete the files older than numDays days from the application cache
 * 0 means all files.
 */


    fun clearCache(context: Context, numDays: Int) {
        // Log.i(FragmentActivity.TAG, String.format("Starting cache prune, deleting files older than %d days", numDays))
        val numDeletedFiles = clearCacheFolder(context.cacheDir, numDays)
        //Log.i(FragmentActivity.TAG, String.format("Cache pruning completed, %d files deleted", numDeletedFiles))
    }


}
